import pytest
import os,sys,inspect
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_rdf():
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [4, 4, 4])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms
    rdf, r = sys.calculate_rdf()

    args = np.argsort(rdf)
    assert(np.round(r[args][-1], decimals=2) == 0.83)

    #test for an fcc type
    atoms, boxdims = pcs.make_crystal('fcc', repetitions = [4, 4, 4])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    rdf, r = sys.calculate_rdf()

    args = np.argsort(rdf)
    assert(np.round(r[args][-1], decimals=2) == 0.69)



import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs


def test_q_4():
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [4, 4, 4])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    #sys.get_neighbors(method = 'voronoi')
    sys.find_neighbors(method = 'cutoff', cutoff=0.9)
    sys.calculate_q([4, 6], averaged=True)
    atoms = sys.atoms
    #assert np.round(atoms[0].q[2], decimals=2) == 0.51
    assert np.round(atoms[0].get_q(4, averaged=True), decimals=2) == 0.51
    assert np.round(atoms[0].get_q([4, 6])[0], decimals=2) == 0.51
    assert np.round(atoms[0].get_q([4, 6], averaged=True)[1], decimals=2) == 0.63
    assert np.round(atoms[0].get_q([4, 6]), decimals=2)[0] == 0.51
    assert np.round(atoms[0].get_q([4, 6], averaged=True)[1], decimals=2) == 0.63

    #now change the q4 values
    atoms[0].set_q(4, .23)
    atoms[0].set_q(4, .23, averaged=True)
    assert np.round(atoms[0].get_q(4), decimals=2) == 0.23
    assert np.round(atoms[0].get_q(4, averaged=True), decimals=2) == 0.23

    atoms[0].set_q([4, 6], [.23, .46])
    atoms[0].set_q([4, 6], [.23, .46], averaged=True)
    assert np.round(atoms[0].get_q(4), decimals=2) == 0.23
    assert np.round(atoms[0].get_q(4, averaged=True), decimals=2) == 0.23
    assert np.round(atoms[0].get_q(6), decimals=2) == 0.46
    assert np.round(atoms[0].get_q(6, averaged=True), decimals=2) == 0.46


def test_q_access():
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [4, 4, 4])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    #sys.get_neighbors(method = 'voronoi')
    sys.find_neighbors(method = 'cutoff', cutoff=0.9)
    sys.calculate_q([4, 6], averaged=True)
    atoms = sys.atoms

    assert np.round(sys.get_custom(atoms[1], ["aq4"]), decimals=2)[0] == 0.51
    assert np.round(sys.get_custom(atoms[1], ["q4"]), decimals=2)[0] == 0.51  


import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_triclinic():
    sys = pc.System()
    sys.read_inputfile('tests/conf.primitive.bcc.supercell.dump')
    sys.find_neighbors(method = 'cutoff', cutoff=1.2)
    atoms = sys.atoms
    neighs = atoms[0].neighbors
    assert len(neighs) == 14

    sys.find_neighbors(method = 'cutoff', cutoff=0.9)
    atoms = sys.atoms
    neighs = atoms[0].neighbors
    assert len(neighs) == 8

def test_triclinic_frames():

    os.system("cat tests/conf.primitive.bcc.supercell.dump tests/conf.primitive.bcc.supercell.dump > tests/bcc.prim.dat")
    sys3 = pc.System()
    sys3.read_inputfile("tests/bcc.prim.dat")
    sys3.find_neighbors(method = 'cutoff', cutoff=0.9)
    atoms = sys3.atoms
    neighs = atoms[0].neighbors
    assert len(neighs) == 8



import pytest
import os,sys,inspect
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs
from ase.build import bulk

#test conventional cna for fcc
def test_cna_cutoff():
    atoms, box = pcs.make_crystal(structure="fcc", repetitions=(7,7,7), lattice_constant=4.00, noise=0.01)
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms
    res = sys.calculate_cna(lattice_constant=4.00)
    assert res["fcc"] == 7*7*7*4

def test_cna_a1():
    atoms, box = pcs.make_crystal(structure="fcc", repetitions=(7,7,7), lattice_constant=4.00, noise=0.01)
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms
    res = sys.calculate_cna(lattice_constant=None)
    assert res["fcc"] == 7*7*7*4

#now test adaptive
def test_cna_adaptive():
    atoms, box = pcs.make_crystal(structure="fcc", repetitions=(7,7,7), lattice_constant=4.00, noise=0.1)
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms
    sys.calculate_cna(lattice_constant=None)
    atoms = sys.atoms
    assert atoms[0].structure == 1

    atoms, box = pcs.make_crystal(structure="hcp", repetitions=(7,7,7), lattice_constant=4.00, noise=0.1)
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms
    sys.calculate_cna(lattice_constant=None)
    atoms = sys.atoms
    assert atoms[0].structure == 2

    atoms, box = pcs.make_crystal(structure="bcc", repetitions=(7,7,7), lattice_constant=4.00, noise=0.1)
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms
    sys.calculate_cna(lattice_constant=None)
    atoms = sys.atoms
    assert atoms[0].structure == 3

def test_cna_diamond():
    atoms, box = pcs.make_crystal(structure="diamond", repetitions=(7,7,7), lattice_constant=4.00, noise=0.1)
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms
    sys.identify_diamond()
    atoms = sys.atoms
    assert atoms[0].structure == 5


def test_ase_bulks():
    
    al_fcc = bulk("Al")
    fe_bcc = bulk("Fe")
    ti_hcp = bulk("Ti")

    sys = pc.System()
    sys.read_inputfile(al_fcc, format="ase")
    cna = sys.calculate_cna()
    assert cna["fcc"] == 1

    sys = pc.System()
    sys.read_inputfile(fe_bcc, format="ase")
    cna = sys.calculate_cna()
    assert cna["bcc"] == 1

    sys = pc.System()
    sys.read_inputfile(ti_hcp, format="ase")
    cna = sys.calculate_cna()
    assert cna["hcp"] == 2


import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

#@profile
def test_basic_system():
    #basic system tests
    sys = pc.System()
    #sys.set_box([[0,1],[0,1],[0,1]])
    #assert sys.get_box() == [[0,1],[0,1],[0,1]]
    #sys.read_inputfile("conf.dump")
    #del sys
    pc.test()


#@profile
def test_system_read():
    sys = pc.System()
    sys.read_inputfile('tests/conf.dump')
    atoms = sys.atoms
    assert len(atoms) == 500

    #check box
    assert sys.box == [[18.85618, 0.0, 0.0], [0.0, 18.86225, 0.0], [0.0, 0.0, 19.01117]]
    #check few atoms
    filtered_atoms = [ atom for atom in atoms if atom.id == 204]
    assert filtered_atoms[0].pos == [-0.10301, -6.35752, -6.44787]

    #now check the same for zipped file
    sys = pc.System()
    sys.read_inputfile('tests/conf.dump.gz')
    atoms = sys.atoms
    assert len(atoms) == 500

    #check box
    assert sys.box == [[18.85618, 0.0, 0.0], [0.0, 18.86225, 0.0], [0.0, 0.0, 19.01117]]

    #check few atoms
    filtered_atoms = [ atom for atom in atoms if atom.id == 204]
    assert filtered_atoms[0].pos == [-0.10301, -6.35752, -6.44787]
    #del sys

    sys = pc.System()
    with pytest.raises(IOError):
        sys.read_inputfile('tests/ahdkklc.dump')

#@profile
def test_system_atom_access():
    #create some atoms
    atoms, boxdims = pcs.make_crystal('bcc')
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    #fetch an atom
    xoxo = sys.get_atom(0)
    assert xoxo.pos == [0, 0, 0]

    #change pos pf ayom
    xoxo.pos = [1.1, 2.1, 3.1]
    sys.set_atom(xoxo)

    atom = sys.get_atom(0)
    assert atom.pos == [1.1, 2.1, 3.1]
    #sys.atoms = atoms
    #atom = sys.get_atom(0)
    #assert atom.pos == [0.1, 0.1, 0.1]
    #del sys



import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs


def test_voro_props():
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [10, 10, 10])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms


    sys.find_neighbors(method = 'voronoi')
    sys.calculate_vorovector()
    atoms = sys.atoms
    atom = atoms[0]
    #v = atom.vorovector
    #assert v == [0,6,0,8]



import pytest
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_chiparamsbcc():
    atoms, box = pcs.make_crystal('bcc', repetitions=[3,3,3], lattice_constant=4)
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms
    sys.find_neighbors(method='cutoff', cutoff=0)
    sys.calculate_chiparams()
    atoms = sys.atoms
    chip = atoms[2].chiparams
    assert chip ==  [3, 0, 0, 0, 36, 12, 0, 36, 0]


def test_chiparamsfcc():
    atoms, box = pcs.make_crystal('fcc', repetitions=[5,5,5], lattice_constant=4)
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms
    sys.find_neighbors(method='cutoff', cutoff=0)
    sys.calculate_chiparams()
    atoms = sys.atoms
    chip = atoms[2].chiparams
    assert chip ==  [6, 0, 0, 0, 24, 12, 0, 24, 0]

def test_chiparamshcp():
    atoms, box = pcs.make_crystal('hcp', repetitions=[3,3,3], lattice_constant=4)
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms
    sys.find_neighbors(method='cutoff', cutoff=0)
    sys.calculate_chiparams()
    atoms = sys.atoms
    chip = atoms[2].chiparams
    assert chip ==  [3, 0, 6, 0, 21, 12, 0, 24, 0]

def test_chiparamsdia():
    atoms, box = pcs.make_crystal('diamond', repetitions=[3,3,3], lattice_constant=4)
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms
    sys.find_neighbors(method='cutoff', cutoff=0)
    sys.calculate_chiparams()
    atoms = sys.atoms
    chip = atoms[2].chiparams
    assert chip ==  [0, 0, 0, 0, 6, 0, 0, 0, 0]



import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_this_file():
    """
    Just a simple function to test if everything works
    """
    assert 1 == 1

def test_get_x():
    atom = pc.Atom(pos=[0,0,0], id=1)
    assert atom.pos == [0,0,0]
    atom.id = 5
    assert atom.id == 5

def test_set_x():
    atom = pc.Atom(pos=[0,0,0], id=1)
    newx = [0,0,2]
    atom.pos = newx
    assert atom.pos == newx

def test_set_type():
    atom = pc.Atom(pos=[0,0,0], id=1, type=1)
    assert atom.type == 1
    atom.type = 2
    assert atom.type == 2

def test_set_solid():
    atom = pc.Atom(pos=[0,0,0], id=1, type=1)
    atom.solid = 1
    assert atom.solid == 1


def test_neighbors():
    atom = pc.Atom(pos=[0,0,0], id=1)
    atom.neighbors = [1,2]
    assert atom.neighbors == [1,2]
    assert atom.coordination == 2
    atom.neighbor_weights = [0.6, 0.4]
    assert atom.neighbor_weights == [0.6, 0.4]



import pytest
import os
import numpy as np
import pyscal.core as pc
#from pyscal.trajectory import Trajectory, hdf_to_dump
from pyscal.trajectory import Trajectory

def test_traj():
	traj = Trajectory("examples/traj.light")
	assert traj.nblocks == 10

	assert len(traj.get_block(0)) == 509

	traj.load(0)
	data = traj.data[0]
	assert data["box"][0][0] ==  -7.34762
	assert data["atoms"]["x"][0] == -4.72745
	traj.unload(0)

def test_timeslice():
	traj = Trajectory("examples/traj.light")
	assert traj.nblocks == 10
	sys = traj[0].to_system()
	assert sys[0].box[0][0] == 18.21922

	aseobj = traj[0].to_ase(species=["Au"])
	assert aseobj[0].positions[0][0] == -4.72745

	od = traj[0].to_dict()
	assert od[0]["box"][0][0] ==  -7.34762

	traj[0].to_file("test.out")
	assert os.path.exists("test.out") == True

	#traj[0].to_hdf("test.hdf")
	#assert os.path.exists("test.hdf") == True

	#hdf_to_dump("test.hdf", "test.dat")
	#assert os.path.exists("test.dat") == True	


import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_q_4():
    atoms, boxdims = pcs.make_crystal('fcc', repetitions = [4, 4, 4])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    #sys.get_neighbors(method = 'voronoi')
    sys.find_neighbors(method = 'cutoff', cutoff=0.9)
    sys.calculate_q(4)
    q = sys.get_qvals(4)
    assert np.round(np.mean(np.array(q)), decimals=2) == 0.19 , "Calculated q4 value is wrong!"

    sys.calculate_q(4, only_averaged=True)
    q = sys.get_qvals(4, averaged=True)
    assert np.round(np.mean(np.array(q)), decimals=2) == 0.19 , "Calculated q4 value is wrong!"

def test_q_4_voro():
    atoms, boxdims = pcs.make_crystal('fcc', repetitions = [4, 4, 4])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    sys.find_neighbors(method = 'voronoi')
    sys.calculate_q(4)
    q = sys.get_qvals(4)
    assert np.round(np.mean(np.array(q)), decimals=2) == 0.19 , "Calculated q4 value is wrong!"

    sys.calculate_q(4, only_averaged=True)
    q = sys.get_qvals(4, averaged=True)
    assert np.round(np.mean(np.array(q)), decimals=2) == 0.19 , "Calculated q4 value is wrong!"

def test_q4_condition():
    atoms, boxdims = pcs.make_crystal('l12', repetitions = [8, 8, 8])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    sys.find_neighbors(method = 'cutoff', cutoff=0.9)
    sys.find_solids()
    sys.calculate_q(4, condition="solid")
    q = sys.get_qvals(4)
    assert np.round(np.mean(np.array(q)), decimals=2) == 0.19 , "Calculated q4 value is wrong!"

    sys.calculate_q(4, clear_condition=True, only_averaged=True)
    q = sys.get_qvals(4, averaged=True)
    assert np.round(np.mean(np.array(q)), decimals=2) == 0.19 , "Calculated q4 value is wrong!"



import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs


def test_q_list():
    #this might take a while, it will find all qs
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [4, 4, 4])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    sys.find_neighbors(method = 'voronoi')

    sys.calculate_q([2, 4], averaged=True)
    q = sys.get_qvals([2, 4], averaged=True)
    #x1 = q[0]
    #x2 = q[1]
    #xx1 = np.round(np.mean(np.array(x1)), decimals=2)
    #xx2 = np.round(np.mean(np.array(x2)), decimals=2)
    #assert xx1 == 0.00
    #assert xx2 == 0.22



import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_cutoff():
    atoms, box = pcs.make_crystal(structure="fcc", lattice_constant=4.07, repetitions=(6,6,6))
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms
    sys.find_neighbors(method="cutoff", cutoff=0)
    sys.set_atom_cutoff(factor=2)
    atoms = sys.atoms
    assert (atoms[0].cutoff - 5.755849198858498) < 1E-5



import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_q_5():
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [4, 4, 4])
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    sys.find_neighbors(method = 'voronoi')

    sys.calculate_q(5, averaged=True)
    q = sys.get_qvals(5, averaged=True)
    assert np.round(np.mean(np.array(q)), decimals=2) == 0.00



import pytest
import os
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs
import pyscal.traj_process as ptp
from ase.build import bulk

def test_lammps_dump():
    sys = pc.System()
    sys.read_inputfile('tests/conf.dump')
    atoms = sys.atoms
    assert len(atoms) == 500

    #check box
    assert sys.box == [[18.85618, 0.0, 0.0], [0.0, 18.86225, 0.0], [0.0, 0.0, 19.01117]]

    #check few atoms
    filtered_atoms = [ atom for atom in atoms if atom.id == 204]
    assert filtered_atoms[0].pos == [-0.10301, -6.35752, -6.44787]

    #now check the same for zipped file
    sys = pc.System()
    sys.read_inputfile('tests/conf.dump.gz')
    atoms = sys.atoms
    assert len(atoms) == 500

    #check box
    assert sys.box == [[18.85618, 0.0, 0.0], [0.0, 18.86225, 0.0], [0.0, 0.0, 19.01117]]

    #check few atoms
    filtered_atoms = [ atom for atom in atoms if atom.id == 204]
    assert filtered_atoms[0].pos == [-0.10301, -6.35752, -6.44787]

def test_neighbors_number():
    atoms, box = pcs.make_crystal(structure="fcc", repetitions=(3,3,3), lattice_constant=4.00)
    sys = pc.System()
    sys.box = box
    sys.atoms = atoms

    sys.find_neighbors(method="number", nmax=8)
    atoms = sys.atoms
    nns = np.mean([atom.coordination for atom in atoms])
    assert nns == 8

    sys.find_neighbors(method="number", nmax=12)
    atoms = sys.atoms
    nns = np.mean([atom.coordination for atom in atoms])
    assert nns == 12


def test_scaled():
    sys = pc.System()
    sys.read_inputfile('tests/conf.bcc.scaled.dump')
    atoms = sys.atoms
    assert len(atoms) == 2
    assert atoms[1].pos[0] == 1.0

def test_poscar():
    sys = pc.System()
    sys.read_inputfile('tests/POSCAR', format='poscar')
    atoms = sys.atoms
    assert len(atoms) == 42

    #now assert atoms of different types
    type1 = len([atom for atom in atoms if atom.type == 1])
    type2 = len([atom for atom in atoms if atom.type == 2])
    assert type1 == 4
    assert type2 == 38

    #now test the coorfinates of the atom
    assert sys.box == [[6.6146*2.83746000000000, 0.0, 0.0], [0.0, 3.16228*2.83746000000000, 0.0], [0.0, 0.0, 1.0*2.83746000000000]]

    #now check coordinates of first atom
    #assert atoms[0].pos == [0.020389021322710754, 8.8981229427339, 0.0005978263145216028]


def test_others():
    sys = pc.System()
    sys.read_inputfile('tests/conf.fcc.Al.dump')
    sys.to_file('tests/prim1', format="poscar", species=['Fe'])

    #try reading in
    sys = pc.System()
    sys.read_inputfile('tests/prim1', format="poscar")
    #aseobj = ptp.con
    ase = bulk('Cu', 'fcc', a=3.6).repeat((3,3,3))
    sys = pc.System()
    sys.read_inputfile(ase, format="ase")

    ase = bulk('Cu', 'fcc', a=3.6, cubic=True).repeat((3,3,3))
    sys = pc.System()
    sys.read_inputfile(ase, format="ase")

def test_poscar_write():
    sys = pc.System()
    sys.read_inputfile('tests/POSCAR', format='poscar')
    atoms = sys.atoms
    assert len(atoms) == 42

    ptp.write_file(sys, 'tests/POSCARtest', format="poscar", species=["Al", "Cu"])
    sys = pc.System()
    sys.read_inputfile('tests/POSCARtest', format='poscar')
    atoms = sys.atoms
    assert len(atoms) == 42
    #now assert atoms of different types
    type1 = len([atom for atom in atoms if atom.type == 1])
    type2 = len([atom for atom in atoms if atom.type == 2])
    assert type1 == 4
    assert type2 == 38

    #now test the coorfinates of the atom
    assert sys.box == [[6.6146*2.83746000000000, 0.0, 0.0], [0.0, 3.16228*2.83746000000000, 0.0], [0.0, 0.0, 1.0*2.83746000000000]]



import pytest
import os,sys,inspect
import numpy as np
import pyscal.core as pc
import pyscal.crystal_structures as pcs

def test_cs_ges():
    atoms, boxdims = pcs.make_crystal('bcc', repetitions = [4, 4, 4], lattice_constant=4.00)
    sys = pc.System()
    sys.box = boxdims
    sys.atoms = atoms

    sys.calculate_centrosymmetry(nmax=8)
    atoms = sys.atoms
    q = [atom.centrosymmetry for atom in atoms]
    assert np.round(np.mean(np.array(q)), decimals=2) == 0.00

    sys.calculate_centrosymmetry(nmax=12)
    atoms = sys.atoms
    q = [atom.centrosymmetry for atom in atoms]
    assert np.round(np.mean(np.array(q)), decimals=2) > 0.00


def test_csm_ovito():
    sys = pc.System()
    sys.read_inputfile("tests/bcc.csm.dump", customkeys=["Centrosymmetry"])    
    sys.calculate_centrosymmetry(nmax=8)
    atoms = sys.atoms
    ind = np.random.randint(0, len(atoms))
    assert np.abs(atoms[ind].centrosymmetry - float(atoms[ind].custom["Centrosymmetry"])) < 1E-5
